@@ -96,9 +96,15 @@ setClass("idenseMatrix", representation("VIRTUAL"),
 setClass("ndenseMatrix", representation(x = "logical", "VIRTUAL"),
 	 contains = c("nMatrix", "denseMatrix"))
 
+## virtual SPARSE ------------
+
+setClass("sparseMatrix", representation("VIRTUAL"), contains = "Matrix")
+
 ## diagonal: has 'diag' slot;  diag = "U"  <--> have identity matrix
 setClass("diagonalMatrix", representation(diag = "character", "VIRTUAL"),
 	 contains = "denseMatrix",
+	 ## TODO??   ^^^^^ -- against DSC'07 talk(3D classes), reconsider
+##>>	 contains = "sparseMatrix", ## which makes "scarceMatrix"  superfluous
 	 validity = function(object) {
 	     d <- object@Dim
 	     if(d[1] != (n <- d[2])) return("matrix is not square")
@@ -112,10 +118,6 @@ setClass("diagonalMatrix", representation(diag = "character", "VIRTUAL"),
 	 prototype = prototype(diag = "N")
 	 )
 
-## virtual SPARSE ------------
-
-setClass("sparseMatrix", representation("VIRTUAL"), contains = "Matrix")
-
 ## sparse matrices in Triplet representation (dgT, lgT, ..):
 setClass("TsparseMatrix", representation(i = "integer", j = "integer", "VIRTUAL"),
 	 contains = "sparseMatrix",
@@ -258,9 +260,9 @@ setClass("nspMatrix",
 
 ## 'diagonalMatrix' already has validity checking
 ## diagonal, numeric matrices;	    "d*" has 'x' slot :
-setClass("ddiMatrix", contains = c("diagonalMatrix", "ddenseMatrix"))# or "dMatrix"
+setClass("ddiMatrix", contains = c("diagonalMatrix", "dMatrix"))# or "dMatrix"
 ## diagonal, logical matrices; "ldense*" has 'x' slot :
-setClass("ldiMatrix", contains = c("diagonalMatrix", "ldenseMatrix"))
+setClass("ldiMatrix", contains = c("diagonalMatrix", "lMatrix"))
 
 setClass("corMatrix", representation(sd = "numeric"), contains = "dpoMatrix",
 	 validity = function(object) {
@@ -325,6 +327,10 @@ setClass("dsCMatrix",
 	 validity = function(object) .Call(tCMatrix_validate, object)
 	 )
 
+if(FALSE) ## FIXME ??? Class of positive definite (Csparse symmetric) Matrices:
+setClass("dpCMatrix", contains = "dsCMatrix",
+	 validity = function(object) FIXME("test for pos.definite ??"))
+
 ## numeric, sparse, sorted compressed sparse row-oriented general matrices
 setClass("dgRMatrix",
 	 contains = c("RsparseMatrix", "dsparseMatrix", "generalMatrix"),
@@ -357,13 +363,13 @@ setClass("lgTMatrix",
 ## logical, sparse, triplet triangular matrices
 setClass("ltTMatrix",
 	 contains = c("TsparseMatrix", "lsparseMatrix", "triangularMatrix"),
-	 validity = function(object) .Call(xTMatrix_validate, object)
+	 validity = function(object) .Call(tTMatrix_validate, object)
 	 )
 
 ## logical, sparse, triplet symmetric matrices
 setClass("lsTMatrix",
 	 contains = c("TsparseMatrix", "lsparseMatrix", "symmetricMatrix"),
-	 validity = function(object) .Call(xTMatrix_validate, object)
+	 validity = function(object) .Call(tTMatrix_validate, object)
 	 )
 
 ## logical, sparse, sorted compressed sparse column-oriented general matrices
@@ -471,13 +477,13 @@ setClass("igTMatrix",
 ## integer, sparse, triplet triangular matrices
 setClass("itTMatrix",
 	 contains = c("TsparseMatrix", "isparseMatrix", "triangularMatrix"),
-	 validity = function(object) .Call(xTMatrix_validate, object)
+	 validity = function(object) .Call(tTMatrix_validate, object)
 	 )
 
 ## integer, sparse, triplet symmetric matrices
 setClass("isTMatrix",
 	 contains = c("TsparseMatrix", "isparseMatrix", "symmetricMatrix"),
-	 validity = function(object) .Call(xTMatrix_validate, object)
+	 validity = function(object) .Call(tTMatrix_validate, object)
 	 )
 
 ## integer, sparse, sorted compressed sparse column-oriented general matrices
@@ -533,6 +539,10 @@ setClass("pMatrix", representation(perm = "integer"),
 	     TRUE
 	 })
 
+## "scarce", those "sparse" in a more common sense,
+##  -----   e.g., do not coerce these to dense needlessly :
+setClassUnion("scarceMatrix", members = c("sparseMatrix", "diagonalMatrix"))
+
 
 ### Factorization classes ---------------------------------------------
 
@@ -676,6 +686,7 @@ setClassUnion("replValue", members = c("numeric", "logical", "complex", "raw"))
 ### Sparse Vectors ---- here use 1-based indexing ! -----------
 setClass("sparseVector",
          representation(length = "integer", i = "integer", "VIRTUAL"),
+	 prototype = prototype(length = 0L),
          validity = function(object) {
              n <- object@length
              if(any(object@i < 1L) || any(object@i > n))
