@@ -35,10 +35,8 @@ setGeneric("lu", function(x, ...) standardGeneric("lu"))
 ## base::rcond() has additional argument 'triangular'
 ## which should *not* be part of the signature
 setGeneric("rcond", function(x, norm, ...) standardGeneric("rcond"),
-	   useAsDefault =
-           ## for now:
-           if(existsFunction("rcond", where=baseenv()))
-           ## later: if(exists(getRversion() >= "2.7.0")
+	   signature = c("x", "norm"),
+	   useAsDefault = if(getRversion() >= "2.7.0")
 	   function(x, norm, ...) base::rcond(x, norm=norm, ...))
 
 
@@ -83,58 +81,9 @@ setGeneric("forceSymmetric",
 	   function(x, uplo) standardGeneric("forceSymmetric"))
 
 
+if(all(names(formals(colSums)) != "...")) { ## maybe unneeded in future
 
-###---- Group Generics ----
-
-if (getRversion() < "2.6.0") {
-## The following are **WORKAROUND** s currently needed for all non-Primitives:
-
-## [The following is more future-proof than direct  setGeneric(.) calls:
-## FIX (in R!) : "trunc" should really be in Math, but we try both for the time
-
-## "Math"
-for(fname in intersect(getGroupMembers("Math"),
-		       c("log", "log2", "log10", "logb", "log1p", "expm1",
-			 "gamma", "lgamma", "digamma", "trigamma",
-			 "cummax", "cummin", "trunc")))
-    if(!is.primitive(get(fname))) setGeneric(fname, group="Math")
-
-## "Math2"
-for(fname in intersect(getGroupMembers("Math2"),
-		       c("round", "signif", "trunc")))
-    if (!is.primitive(get(fname))) setGeneric(fname, group="Math2")
-
-## "Summary"
-
-    ## --- some hoop jumping that is needed for R versions <= 2.5.x
-
-.max_def <- function(x, ..., na.rm = FALSE) base::max(x, ..., na.rm = na.rm)
-.min_def <- function(x, ..., na.rm = FALSE) base::min(x, ..., na.rm = na.rm)
-.range_def <- function(x, ..., na.rm = FALSE) base::range(x, ..., na.rm = na.rm)
-.prod_def <- function(x, ..., na.rm = FALSE) base::prod(x, ..., na.rm = na.rm)
-.sum_def <- function(x, ..., na.rm = FALSE) base::sum(x, ..., na.rm = na.rm)
-.any_def <- function(x, ..., na.rm = FALSE) base::any(x, ..., na.rm = na.rm)
-.all_def <- function(x, ..., na.rm = FALSE) base::all(x, ..., na.rm = na.rm)
-
-setGeneric("max", function(x, ..., na.rm = FALSE) standardGeneric("max"),
-           useAsDefault = .max_def, group = "Summary")
-setGeneric("min", function(x, ..., na.rm = FALSE) standardGeneric("min"),
-           useAsDefault = .min_def, group="Summary")
-setGeneric("range", function(x, ..., na.rm = FALSE) standardGeneric("range"),
-           useAsDefault = .range_def, group="Summary")
-setGeneric("prod", function(x, ..., na.rm = FALSE) standardGeneric("prod"),
-           useAsDefault = .prod_def, group="Summary")
-setGeneric("sum", function(x, ..., na.rm = FALSE) standardGeneric("sum"),
-           useAsDefault = .sum_def, group="Summary")
-setGeneric("any", function(x, ..., na.rm = FALSE) standardGeneric("any"),
-           useAsDefault = .any_def, group="Summary")
-setGeneric("all", function(x, ..., na.rm = FALSE) standardGeneric("all"),
-           useAsDefault = .all_def, group="Summary")
-
-} # end{hoop jumping etc for R versions <= 2.5.1 }
-
-
-## Add '...' so our methods can add  'sparseResult':
+    ## Add '...' so our methods can add  'sparseResult':
 setGeneric("colSums",
 	   def = function(x, na.rm = FALSE, dims = 1, ...) standardGeneric("colSums"),
 	   useAsDefault = function(x,  na.rm = FALSE, dims = 1, ...)
@@ -151,3 +100,4 @@ setGeneric("rowMeans",
 	   def = function(x, na.rm = FALSE, dims = 1, ...) standardGeneric("rowMeans"),
 	   useAsDefault = function(x,  na.rm = FALSE, dims = 1, ...)
 	   base::rowMeans(x, na.rm=na.rm, dims=dims))
+}
