@@ -122,6 +122,7 @@ replCmat <- function (x, i, j, ..., value)
     dn <- dimnames(x)
     iMi <- missing(i)
     jMi <- missing(j)
+    spV <- is(value, "sparseVector")
     na <- nargs()
     if(na == 3) { ## "vector (or 2-col) indexing"  M[i] <- v
 	x <- as(x, "TsparseMatrix")
@@ -154,16 +155,18 @@ replCmat <- function (x, i, j, ..., value)
     ## keep "symmetry" if changed here:
     x.sym <- extends(clDx, "symmetricMatrix")
     if(x.sym) { ## only half the indices are there..
+        mkArray <- if(spV) # TODO: room for improvement
+            function(v, dim) spV2M(v, dim[1],dim[2]) else array
 	x.sym <-
 	    (dind[1] == dind[2] && all(i1 == i2) &&
-	     (lenRepl == 1 || isSymmetric(array(value, dim=dind))))
+	     (lenRepl == 1 || isSymmetric(mkArray(value, dim=dind))))
 	## x.sym : result is *still* symmetric
 	x <- .Call(Csparse_symmetric_to_general, x) ## but do *not* redefine clx!
     }
     else if((tri.x <- extends(clDx, "triangularMatrix"))) {
         xU <- x@uplo == "U"
 	r.tri <- ((any(dind == 1) || dind[1] == dind[2]) &&
-		  all(if(xU) i1 <= i2 else i2 <= i1))
+		  if(xU) max(i1) <= min(i2) else max(i2) <= min(i1))
 	if(r.tri) { ## result is *still* triangular
             if(any(i1 == i2)) # diagonal will be changed
                 x <- diagU2N(x) # keeps class (!)
@@ -183,9 +186,10 @@ replCmat <- function (x, i, j, ..., value)
         ## need indices instead of just 'sel', for, e.g.,  A[2:1, 2:1] <- v
 	non0 <- cbind(match(x@i[sel], i1),
 		      match(xj [sel], i2)) - 1L
-	iN0 <- 1L + encodeInd(non0, nr = dind[1])
+	iN0 <- 1L + .Call(m_encodeInd, non0, di = dind)
 
-        has0 <- any(value[!is.na(value)] == 0)
+        has0 <-
+            if(spV) length(value@i) < lenV else any(value[!is.na(value)] == 0)
         if(lenV < lenRepl)
             value <- rep(value, length = lenRepl)
 	## Ideally we only replace them where value != 0 and drop the value==0
@@ -195,7 +199,7 @@ replCmat <- function (x, i, j, ..., value)
 	## if (lenRepl == 1) and v0 is TRUE, the following is not doing anything
 	##-  --> ./dgTMatrix.R	and its	 replTmat()
 	## x@x[sel[!v0]] <- value[!v0]
-        x@x[sel] <- value[iN0]
+        x@x[sel] <- as.vector(value[iN0])
         if(has0) x <- .Call(Csparse_drop, x, 0)
 
 	return(if(x.sym) as_CspClass(x, clx) else x)
@@ -214,7 +218,7 @@ replCmat <- function (x, i, j, ..., value)
 	x[i,j] <- value
 
     if(any(is0(x@x))) ## drop all values that "happen to be 0"
-	drop0(x, clx)
+	drop0(x)
     else as_CspClass(x, clx)
 }
 
@@ -230,6 +234,19 @@ setReplaceMethod("[", signature(x = "CsparseMatrix", i = "index", j = "index",
 				value = "replValue"),
                  replCmat)
 
+### When the RHS 'value' is  a sparseVector, now can use  replCmat  as well
+setReplaceMethod("[", signature(x = "CsparseMatrix", i = "missing", j = "index",
+				value = "sparseVector"),
+		 replCmat)
+
+setReplaceMethod("[", signature(x = "CsparseMatrix", i = "index", j = "missing",
+				value = "sparseVector"),
+		 replCmat)
+
+setReplaceMethod("[", signature(x = "CsparseMatrix", i = "index", j = "index",
+				value = "sparseVector"),
+		 replCmat)
+
 ## A[ ij ] <- value,  where ij is (i,j) 2-column matrix
 setReplaceMethod("[", signature(x = "CsparseMatrix", i = "matrix", j = "missing",
 				value = "replValue"),
@@ -336,7 +353,7 @@ setMethod("band", "CsparseMatrix",
 	      k2 <- as.integer(k2[1])
 	      dd <- dim(x); sqr <- dd[1] == dd[2]
 	      stopifnot(-dd[1] <= k1, k1 <= k2, k2 <= dd[1])
-	      r <- .Call(Csparse_band, x, k1, k2)
+	      r <- .Call(Csparse_band, diagU2N(x), k1, k2)
 	      if(sqr && k1 * k2 >= 0) ## triangular
 		  as(r, paste(.M.kind(x), "tCMatrix", sep=''))
 	      else if (k1 < 0  &&  k1 == -k2  && isSymmetric(x)) ## symmetric
@@ -348,7 +365,7 @@ setMethod("band", "CsparseMatrix",
 setMethod("diag", "CsparseMatrix",
 	  function(x, nrow, ncol) {
               ## "FIXME": could be more efficient; creates new ..CMatrix:
-	      dm <- .Call(Csparse_band, x, 0, 0)
+	      dm <- .Call(Csparse_band, diagU2N(x), 0, 0)
 	      dlen <- min(dm@Dim)
 	      ind1 <- dm@i + 1L	# 1-based index vector
 	      if (is(dm, "nMatrix")) {
