@@ -32,27 +32,38 @@ Diagonal <- function(n, x = NULL)
     }
 }
 
-## Pkg 'spdep' had (relatively slow) versions of this as_dsCMatrix_I()
-.symDiagonal <- function(n, x = rep.int(1,n), uplo = "U") {
+.sparseDiagonal <- function(n, x = rep.int(1,n), uplo = "U", shape = "t") {
     stopifnot(n == (n. <- as.integer(n)), (n <- n.) >= 0)
     if((lx <- length(x)) == 1) x <- rep.int(x, n)
     else if(lx != n) stop("length(x) must be 1 or n")
-    cls <-
-        if(is.double(x)) "dsCMatrix"
-        else if(is.logical(x)) "lsCMatrix"
-        else { ## for now
-            storage.mode(x) <- "double"
-            "dsCMatrix"
-        }
-    new(cls, Dim = c(n,n), x = x, uplo = uplo,
-        i = if(n) 0:(n - 1L) else integer(0), p = 0:n)
+    stopifnot(is.character(shape), nchar(shape) == 1,
+	      any(shape == c("t","s","g"))) # triangular / symmetric / general
+    kind <-
+	if(is.double(x)) "d"
+	else if(is.logical(x)) "l"
+	else { ## for now
+	    storage.mode(x) <- "double"
+	    "d"
+	}
+    new(paste(kind, shape, "CMatrix", sep=''),
+	Dim = c(n,n), x = x, uplo = uplo,
+	i = if(n) 0:(n - 1L) else integer(0), p = 0:n)
 }
 
+## Pkg 'spdep' had (relatively slow) versions of this as_dsCMatrix_I()
+.symDiagonal <- function(n, x = rep.int(1,n), uplo = "U")
+    .sparseDiagonal(n, x, uplo, shape = "s")
+
+## instead of   diagU2N(as(Diagonal(n), "CsparseMatrix")), diag = "N" in any case:
+.trDiagonal <- function(n, x = rep.int(1,n), uplo = "U")
+    .sparseDiagonal(n, x, uplo, shape = "t")
+
+
 ### This is modified from a post of Bert Gunter to R-help on  1 Sep 2005.
 ### Bert's code built on a post by Andy Liaw who most probably was influenced
 ### by earlier posts, notably one by Scott Chasalow on S-news, 16 Jan 2002
 ### who posted his bdiag() function written in December 1995.
-if(FALSE)##--- currently unused:
+if(FALSE)##--- no longer used:
 .bdiag <- function(lst) {
     ### block-diagonal matrix [a dgTMatrix] from list of matrices
     stopifnot(is.list(lst), length(lst) >= 1)
@@ -74,23 +85,73 @@ if(FALSE)##--- currently unused:
     }
     r
 }
-### Doug Bates needed something like bdiag() for lower-triangular
-### (Tsparse) Matrices and provided a much more efficient implementation:
+## expand(<mer>) needed something like bdiag() for lower-triangular
+## (Tsparse) Matrices; hence Doug Bates provided a much more efficient
+##  implementation for those; now extended and generalized:
 .bdiag <- function(lst) {
-    ### block-diagonal matrix [a dgTMatrix] from list of matrices
+    ## block-diagonal matrix [a dgTMatrix] from list of matrices
     stopifnot(is.list(lst), (nl <- length(lst)) >= 1)
 
     Tlst <- lapply(lapply(lst, Matrix:::as_Csp2), # includes "diagU2N"
-                   as, "TsparseMatrix")
-
+		   as, "TsparseMatrix")
     if(nl == 1) return(Tlst[[1]])
     ## else
     i_off <- c(0L, cumsum(sapply(Tlst, nrow)))
     j_off <- c(0L, cumsum(sapply(Tlst, ncol)))
-    new("dgTMatrix", Dim = c(i_off[nl+1], j_off[nl + 1]),
-	i = unlist(lapply(1:nl, function(k) Tlst[[k]]@i + i_off[k])),
-	j = unlist(lapply(1:nl, function(k) Tlst[[k]]@j + j_off[k])),
-	x = unlist(lapply(Tlst, slot, "x")))
+
+    clss <- sapply(Tlst, class)
+    knds <- substr(clss, 2, 2)
+    sym	 <- knds == "s" # symmetric ones
+    tri	 <- knds == "t" # triangular ones
+    use.n <- any(is.n <- substr(clss,1,1) == "n")
+    if(use.n && !(use.n <- all(is.n)))
+	Tlst[is.n] <- lapply(Tlst[is.n], as, "lMatrix")
+    if(all(sym)) { ## result should be *symmetric*
+	uplos <- sapply(Tlst, slot, "uplo") ## either "U" or "L"
+	tLU <- table(uplos)# of length 1 or 2 ..
+	if(length(tLU) == 1) { ## all "U" or all "L"
+	    useU <- uplos[1] == "U"
+	} else { ## length(tLU) == 2, counting "L" and "U"
+	    useU <- diff(tLU) >= 0
+	    if(useU && (hasL <- tLU[1] > 0))
+		Tlst[hasL] <- lapply(Tlst[hasL], t)
+	    else if(!useU && (hasU <- tLU[2] > 0))
+		Tlst[hasU] <- lapply(Tlst[hasU], t)
+	}
+	if(use.n) { ## return nsparseMatrix :
+	    r <- new("nsTMatrix")
+	} else {
+	    r <- new("dsTMatrix")
+	    r@x <- unlist(lapply(Tlst, slot, "x"))
+	}
+	r@uplo <- if(useU) "U" else "L"
+    }
+    else if(all(tri) && { ULs <- sapply(Tlst, slot, "uplo")##  "U" or "L"
+			  all(ULs[1L] == ULs[-1L]) } ## all upper or all lower
+       ){ ## *triangular* result
+
+	if(use.n) { ## return nsparseMatrix :
+	    r <- new("ntTMatrix")
+	} else {
+	    r <- new("dtTMatrix")
+	    r@x <- unlist(lapply(Tlst, slot, "x"))
+	}
+	r@uplo <- ULs[1L]
+    }
+    else {
+	if(any(sym))
+	    Tlst[sym] <- lapply(Tlst[sym], as, "generalMatrix")
+	if(use.n) { ## return nsparseMatrix :
+	    r <- new("ngTMatrix")
+	} else {
+	    r <- new("dgTMatrix")
+	    r@x <- unlist(lapply(Tlst, slot, "x"))
+	}
+    }
+    r@Dim <- c(i_off[nl+1], j_off[nl + 1])
+    r@i <- unlist(lapply(1:nl, function(k) Tlst[[k]]@i + i_off[k]))
+    r@j <- unlist(lapply(1:nl, function(k) Tlst[[k]]@j + j_off[k]))
+    r
 }
 
 bdiag <- function(...) {
@@ -181,6 +242,7 @@ setAs("diagonalMatrix", "nMatrix",
 	      Dim = from@Dim, Dimnames = from@Dimnames)
       })
 
+setAs("diagonalMatrix", "nsparseMatrix", function(from) as(from, "nMatrix"))
 
 ## Cheap fast substitute for diag() which *does* preserve the mode of x :
 mkDiag <- function(x, n) {
@@ -354,6 +416,29 @@ setReplaceMethod("[", signature(x = "diagonalMatrix", i = "missing",
 				j = "index", value = "replValue"),
 		 function(x,i,j, ..., value) replDiag(x, j=j, value=value))
 
+setReplaceMethod("[", signature(x = "diagonalMatrix", i = "missing", j = "index",
+				value = "scarceMatrix"),
+		 function (x, i, j, ..., value)
+		 callGeneric(x=x, , j=j, value = as(value, "sparseVector")))
+setReplaceMethod("[", signature(x = "diagonalMatrix", i = "index", j = "missing",
+				value = "scarceMatrix"),
+		 function (x, i, j, ..., value)
+		 callGeneric(x=x, i=i, , value = as(value, "sparseVector")))
+setReplaceMethod("[", signature(x = "diagonalMatrix", i = "index", j = "index",
+				value = "scarceMatrix"),
+		 function (x, i, j, ..., value)
+		 callGeneric(x=x, i=i, j=j, value = as(value, "sparseVector")))
+
+setReplaceMethod("[", signature(x = "diagonalMatrix", i = "missing", j = "index",
+				value = "sparseVector"),
+		 replDiag)
+setReplaceMethod("[", signature(x = "diagonalMatrix", i = "index", j = "missing",
+				value = "sparseVector"),
+		 replDiag)
+setReplaceMethod("[", signature(x = "diagonalMatrix", i = "index", j = "index",
+				value = "sparseVector"),
+		 replDiag)
+
 
 setMethod("t", signature(x = "diagonalMatrix"),
           function(x) { x@Dimnames <- x@Dimnames[2:1] ; x })
@@ -876,6 +961,9 @@ prDiag <-
     invisible(x)
 }
 
+## somewhat consistent with "print" for sparseMatrix :
+setMethod("print", signature(x = "diagonalMatrix"), prDiag)
+
 setMethod("show", signature(object = "diagonalMatrix"),
 	  function(object) {
 	      d <- dim(object)
