@@ -24,9 +24,11 @@ setAs("nsCMatrix", "nsTMatrix",
 	Dim = from@Dim, Dimnames = from@Dimnames)
 
 setAs("nsCMatrix", "dsCMatrix", .nsC2d)
+setAs("nsCMatrix", "dMatrix", .nsC2d)
 setAs("nsCMatrix", "dsparseMatrix", .nsC2d)
 
 setAs("nsCMatrix", "lsCMatrix", .nsC2l)
+setAs("nsCMatrix", "lMatrix", .nsC2l)
 setAs("nsCMatrix", "lsparseMatrix", .nsC2l)
 
 rm(.nsC2d,.nsC2l) # don't even keep "hidden"
@@ -52,10 +54,6 @@ setMethod("triu", "nsCMatrix",
 	      else triu(as(x, "ngCMatrix"), k = k, ...)
 	  })
 
-## FIXME: generalize to "nsparseMatrix" or (class union)  "symmetric sparse"
-setMethod("image", "nsCMatrix",
-	  function(x, ...) image(as(as(x, "dsCMatrix"), "dgTMatrix"), ...))
-
 setMethod("chol", signature(x = "nsCMatrix"),
 	  function(x, pivot=FALSE, ...) stop("temporarily disabled"))## FIXME
 
